package net.meltingwax.anonircd;

import net.meltingwax.anonircd.conf.*;
import net.meltingwax.anonircd.conf.excepts.*;
import net.meltingwax.anonircd.i.*;
import java.io.*;
import java.security.*;


/**
 * The Driver of the application, which contains the main method.
 */
public class Driver {
    public static void main(String[] args) {
        // Unless the environmental variable ANONIRCD_CONFIG exists, the configuration is
	// assumed to be ~/.anonircd.
	String configPath;
	
	if (System.getenv("ANONIRCD_CONFIG") != null) {
	    configPath = System.getenv("ANONIRCD_CONFIG"); 
	} else {
	    configPath = (new File(System.getProperty("user.home"), ".anonircd")).getPath();
	}

	System.out.println("# Using config in: " + configPath);
	
        // Generate configuration directory if it does not exist.
        if (! new File(configPath).exists()) {
            try {
                ConfigDir.genStarter(configPath);
                System.out.println("Set up config in " + configPath);
            } catch (ConfigGenException e) {
                e.printStackTrace();
                System.out.println(e.getMessage());
            }
        }

        // Create a ConfigProvider based on a config directory.
        ConfigProvider config;

        try {
            config = new ConfigDir(configPath, new ConfigDefaults());
        } catch (ConfigLoadException e) {
            e.getOriginalException().printStackTrace();
            return;
        }

        // Create the server and start listening.
        Server server = new Server(config);
        server.listen();
    }
}
