package net.meltingwax.anonircd;

import java.io.IOException;
import java.net.Socket;
import java.net.ServerSocket;
import java.util.HashSet;
import java.util.Set;



public class ListenerServer extends Thread {
    private Server server;
    private int port;
    private Set<ConnectionHandler> connections;

    public ListenerServer(Server server, int port) {
        this.server = server;
        this.port = port;
        connections = new HashSet<ConnectionHandler>();
    }

    public void run() {
        ServerSocket serverSocket;

        try {
            serverSocket = new ServerSocket(port);
        } catch (IOException e) {
            System.out.println("Could not listen on port " + port);
            e.printStackTrace();
            return;
        }

        System.out.println("Server started on port " + port);
        System.out.println("Listening for connections...");

        while (true) {
            Socket client;
            try{
                client = serverSocket.accept();
            } catch (IOException e) {
                System.out.println("Accept failed");
                e.printStackTrace();
                continue;
            }

            ConnectionHandler conn;
            try {
                conn = new ConnectionHandler(client, server.config, server, server);
            } catch (IOException e) {
                System.out.println("Accept failed");
                e.printStackTrace();
                continue;
            }

            conn.start();

            System.out.println("Connection made from " + client.getInetAddress().toString());

	    synchronized (connections) {
		connections.add(conn);
	    }
        }
    }

    /**
     * Returns a shallow copy of the connections set. This may contain
     * dead entries, because cleanup is crappy.
     */
    public Set<ConnectionHandler> getConnections() {
	HashSet<ConnectionHandler> copy;
	
	synchronized (connections) {
	    copy = new HashSet<ConnectionHandler>(connections);
	}
	
	return copy;
    }
}
