package net.meltingwax.anonircd;


/**
 * Provides static methods for deciding whether an account has permission to
 * preform a specific acction.
 */
public class PermissionManager {
    // Determines whether the account has permission to assign privileges on
    // a certain channel
    public static boolean canAssign(AuthedAccount account, String channelName) {
        return (account.getIsSysOp()
                || account.getChannelPrivilege(channelName) == ChannelPrivilege.OWNER);
    }

    // Determines whether the account has permission to ban on a given channel.
    public static boolean canBan(AuthedAccount account, String channelName) {
        ChannelPrivilege chanPriv = account.getChannelPrivilege(channelName);

        return (account.getIsSysOp()
                || chanPriv == ChannelPrivilege.OWNER
                || chanPriv == ChannelPrivilege.OP);

    }

    // Determines whether the account has permission to kick on a given channel.
    public static boolean canKick(AuthedAccount account, String channelName) {
        ChannelPrivilege chanPriv = account.getChannelPrivilege(channelName);

        return (account.getIsSysOp()
                || chanPriv == ChannelPrivilege.OWNER
                || chanPriv == ChannelPrivilege.OP
                || chanPriv == ChannelPrivilege.HALF_OP);
    }

    // Determines whether the account has permission to notice the whole server.
    public static boolean canNotice(AuthedAccount account) {
        return account.getIsSysOp();
    }

    // Determines whether the account has permission to reveal on a given channel.
    public static boolean canReveal(AuthedAccount account, String channelName) {
        ChannelPrivilege chanPriv = account.getChannelPrivilege(channelName);

        return (account.getIsSysOp()
                || chanPriv == ChannelPrivilege.OWNER
                || chanPriv == ChannelPrivilege.OP
                || chanPriv == ChannelPrivilege.HALF_OP);
    }
}
