
package net.meltingwax.anonircd;


import net.meltingwax.anonircd.bots.*;
import net.meltingwax.anonircd.bots.i.*;
import net.meltingwax.anonircd.excepts.*;
import net.meltingwax.anonircd.i.*;

import java.security.*;
import java.util.*;


public class Server implements BotProvider, ChannelProvider {
    public ConfigProvider config;
    private Map<String, Channel> channels;
    private Map<String, Bot> bots;
    private ListenerServer listenerServer;

    public Server(ConfigProvider config) {
        this.config = config;

        listenerServer = new ListenerServer(this, config.getPort());
        channels = new HashMap<String, Channel>();
        bots = new HashMap<String, Bot>();
	
        Bot adminServ = new AdminServ(this);
        bots.put(adminServ.getNick().toLowerCase(), adminServ);
    }

    public void listen() {
        listenerServer.start();
        
        try {
            listenerServer.join();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    public Channel getChannel(String channelName) throws IllegalChannelNameException {
        channelName = channelName.toLowerCase();
        Channel channel;

        synchronized (channels) {
            if (! channels.containsKey(channelName)) {
                channel = new Channel(config, channelName);  // throws IllegalChannelNameException
                channels.put(channelName, channel);
            } else {
                channel = channels.get(channelName);
            }
        }

        return channel;
    }

    public Bot getBot(String botNick) throws BotNotFoundException {
        botNick = botNick.toLowerCase();

        if (bots.containsKey(botNick)) {
            return bots.get(botNick);
        } else {
            throw new BotNotFoundException(botNick);
        }
    }

    public Map<String, Channel> getAllChannels() {
        synchronized (channels) {
            return new HashMap<String, Channel>(channels);
        }
    }

    public void sendSystemNotice(String message) {
	for (ConnectionHandler connection : listenerServer.getConnections()) {
	    if (connection.isParticipantAlive()) {
		connection.onSystemNotice(message);
	    }
	}
    }
}
