package net.meltingwax.anonircd.bots;

import net.meltingwax.anonircd.*;
import net.meltingwax.anonircd.bots.excepts.*;
import org.jdom2.*;
import org.jdom2.input.*;
import java.io.*;
import java.util.*;


/**
 * This is a helper class for AdminServ. Although it is in the bots package, it
 * it not actually a bot itself (it does not satisfy the interface).
 *
 * This class requires AdminServHelp.xml to exist in the same directory; this is the
 * file used to store information about the commands.
 */
public class AdminServHelp {
    private Document xmlDoc;
    
    /**
     * Will raise an Error if the XML document cannot be read or parsed correctly.
     */
    public AdminServHelp() {
	String currentDir = getClass().getPackage().getName().replace(".", "/");       
        String xmlFileName =  currentDir + "/AdminServHelp.xml";
        InputStream xmlStream = getClass().getClassLoader().getResourceAsStream(xmlFileName);

	// Parse the XML document.
        SAXBuilder builder = new SAXBuilder();

        try {
            this.xmlDoc = builder.build(xmlStream);
        } catch (JDOMException e) {
            throw new Error(e);
        } catch (IOException e) {
            throw new Error(e);
        }
    }

    /**
     * Generate the usage for a specific command.
     */
    public List<String> getCommandUsage(String command) throws CommandNotFoundException {
	// Search for <command> in XML that matches this command name,
	// throw exception if not found.
        Element element = null;
        List<Element> children = xmlDoc.getRootElement().getChildren("command");

        for (Element tmpElement : children) {
            if (tmpElement.getAttributeValue("name").toUpperCase().equals(command.toUpperCase())) {
                element = tmpElement;
                break;
            }
        }

        if (element == null) {
            throw new CommandNotFoundException();
        }

	// Add usage information from element
        List<String> lines = new ArrayList<String>();
	lines.addAll(getCommandUsage(element));
	
	// Add privilege information if necessary
	String[] args = element.getAttributeValue("args").split(" ");	
	boolean hasPrivilegeArg = Utils.stringArrayContains(args, "privilege");
	
	if (hasPrivilegeArg) {
	    lines.add("");
	    lines.addAll(getChannelPrivilegeInfo());
	}

	return lines;
    }

    /**
     * Transforms an XML command element into a sendable list of
     * line strings.
     */
    private List<String> getCommandUsage(Element element) {
	List<String> lines = new ArrayList<String>();

	// Add header aline
	String args = element.getAttributeValue("args");
	String line = element.getAttributeValue("name").toUpperCase();
	if (args.length() > 0) {
	    for (String arg : args.split(" ")) {
		line += " <" + arg.toLowerCase() + ">";
	    }
	}
	lines.add(line);
	
	// Add description line
	lines.add(element.getTextNormalize());

	return lines;
    }

    /**
     * Generates a sendable list of lines about the available privileges an
     * admin user can have on a channel.
     */
    private List<String> getChannelPrivilegeInfo() {
	List<String> lines = new ArrayList<String>();	
	
	lines.add("Levels of privilege admin accounts can have on channels are: ");
	lines.add(Utils.join(", ", AuthedAccount.getChannelPrivilegeNames()));
	
	return lines;
    }

    /**
     * Generates full help documentation.
     */
    public List<String> getFullHelp() {
        List<String> lines = new ArrayList<String>();
        
	// Introduction
	lines.add("The following commands are available:");
	lines.add("");
	

	// Add usage information on each command.
	List<Element> children = xmlDoc.getRootElement().getChildren("command");
	
        for (Element element : children) {
	    lines.addAll(getCommandUsage(element));
	    lines.add("");
        }

	// Add the channel privilege information
	lines.addAll(getChannelPrivilegeInfo());
		
        return lines;
    }
}

