package net.meltingwax.anonircd.conf;

import java.math.*;
import java.security.*;


public class ConfigUtils {
    /**
     * Determines whether we have the algorithm available to hash a
     * password.
     */
    public static boolean canHashPasswords() {
        try {
            MessageDigest.getInstance("SHA-1");
            return true;
        } catch (NoSuchAlgorithmException e) {
            return false;
        }
    }

    /**
     * Encrypt a password using a secure hashing method.
     * @param password The password to encrypt
     * @return the encrypted password
     * @throws NoSuchAlgorithmException if SHA-1 encryption is not available
     */
    public static String hashPassword(String password) throws NoSuchAlgorithmException {
        if (password == null) {
            throw new NullPointerException();
        }

        // Iterate 500 times for extra security
        String encrypted = password;

        for (int i=0; i < 500; i++) {
            encrypted = hashString(encrypted);
        }

        return encrypted;
    }
    
    /**
	 * Encrypt a string once using a SHA-1 hash.
	 * @throws NoSuchAlgorithmException
	 */
	public static String hashString(String str) throws NoSuchAlgorithmException {
	    if (str == null) {
	        throw new NullPointerException();
	    }
	
	    MessageDigest crypt = MessageDigest.getInstance("SHA-1");
	
	    crypt.reset();
	    crypt.update(str.getBytes());
	
	    return new BigInteger(1, crypt.digest()).toString(16);
	}
}
