package net.meltingwax.anonircd.i;

import net.meltingwax.anonircd.*;


public interface ChannelParticipant {

    /**
     * Callback for a message from someone else in a channel the participant
     * is in.
     * @param channel
     * @param message
     * @param fromNick
     */
    public void onJoinedChannelMessage(Channel channel, String message, String fromNick);

    /**
     * Callback for a someone else joining a channel the participant is in.
     * @param channel
     */
    public void onJoinedChannelNewChannelParticipant(Channel channel);

    /**
     * Callback for a new topic in a channel the participant is in.
     * @param channel
     */
    public void onJoinedChannelNewTopic(Channel channel);

    /**
     * Callback for for someone else parting a channel the participant is in.
     * @param channel
     */
    public void onJoinedChannelParticipantPart(Channel channel);


    /**
     * This is the method the participant has to handle the event
     * of getting kicked out of a channel.
     */
    public void onKick(Channel channel);


    /**
     * Allows access to the IP address of the participant.
     *
     * @return IP address as a string.
     */
    public String getIPAddress();

    
    /**
     * Decides whether or not the socket with the participant
     * is able to send/receieve information.
     *
     * @return Whether the socket is alive, as defined above.
     */
    public boolean isParticipantAlive();

    /**
     * Returns the nick of this channel participant on a given channel.
     * If the channel is an ID channel, then the nick will be
     * generated according to a certain set of rules.
     */
    public String getOutsideNick(Channel channel);
}
