package net.meltingwax.anonircd.i;

import net.meltingwax.anonircd.*;
import net.meltingwax.anonircd.conf.excepts.*;


public interface ConfigProvider {

    /**
     * Adds a stored IP ban on a channel.
     */
    public void addChannelBan(String channelName, String ipAddr);

    /**
     * Removes a stored IP ban on a channel.
     */
    public void removeChannelBan(String channelName, String ipAddr);

    /**
     * Verify that this login/password combination indeed matches that of
     * a stored authorized account.
     *
     * @param username The username of the account
     * @param password  The password of the account
     */
    public boolean checkAuthCredentials(String username, String password);

    /**
     * Check if a channel is already owned.
     */
    public boolean checkChannelOwned(String channelName);

    /**
     * Check whether an IP is banned from a given channel.
     */
    public boolean checkForChannelBan(String channelName, String ipAddr);

    /**
     * Checks whether a given IP address is a trusted web client.
     */
    public boolean checkForTrustedWebClient(String clientIP);

    /**
     * Reload the configuration settings.
     * @throws ConfigLoadException An error occurred.
     */
    public void reload() throws ConfigLoadException;

    /**
     * Gets an AuthedAccount instance for the given login. If an authed account for
     * this username does not exist, returns null. There should be only one instance
     * of AuthedAccount per username at any given time.
     * Precondition: checkAuthCredentials(login).
     * @param login The username of the account
     * @return The AuthedAccount instance for this account.
     */
    public AuthedAccount getAuthedAccount(String login);

    /**
     * Returns the maximum number of recent channel messages to save for the REVEAL
     * command.
     */
    public int getMaxChannelRecentMessages();

    /**
     * Returns the maximum length of a message.
     */
    public int getMaxMessageLength();

    /**
     * Returns the maximum length of a topic.
     */
    public int getMaxTopicLength();

    /**
     * Returns the MOTD String (with possible \n's)
     */
    public String getMOTD();

    /**
     * Returns the port to listen on.
     */
    public int getPort();

    /**
     * Saves an AuthedAccount. If an account already exists with this
     * username, it is overwritten.
     */
    public void saveAuthedAccount(AuthedAccount account);
    
    /**
     * Saves a new topic for a channel.
     */
	public void saveChannelTopic(String channelName, String topic);
	
	/**
	 * Gets a topic for a channel. If one is not stored, returns null.
	 */
	public String getChannelTopic(String channelName);
}
