package net.meltingwax.anonircd.tests;

import java.security.NoSuchAlgorithmException;
import junit.framework.TestCase;
import net.meltingwax.anonircd.*;
import net.meltingwax.anonircd.conf.*;


/**
 * Tests for the AuthedAccount class.
 */
public class AuthedAccountTest extends TestCase {

    public void testConstructor() {
        new AuthedAccount("abc", "def", true);
    }

    public void testGetIsSysOp() {
        AuthedAccount account;

        account = new AuthedAccount("abc", "def", true);
        assertTrue(account.getIsSysOp());

        account = new AuthedAccount("abc", "def", false);
        assertFalse(account.getIsSysOp());
    }

    public void testGetPasswordHash() {
        AuthedAccount account;

        account = new AuthedAccount("abc", "def", true);
        assertTrue(account.getPasswordHash().equals("def"));
    }

    public void testSetPassword() throws NoSuchAlgorithmException {
        AuthedAccount account;

        account = new AuthedAccount("abc", "def", true);
        account.setPassword("xyz");
        assertFalse(account.getPasswordHash().equals("xyz"));
        assertTrue(account.getPasswordHash().equals(ConfigUtils.hashPassword("xyz")));
    }

    public void testGetUsername() {
        AuthedAccount account = new AuthedAccount("john", "abc", false);

        assertTrue(account.getUsername().equals("john"));
    }

    public void testGetChannelPrivilege() {
        AuthedAccount account = new AuthedAccount("john", "abc", false);
	ChannelPrivilege got = account.getChannelPrivilege("#lounge");

	assertEquals(got, ChannelPrivilege.NONE);
    }

    public void testGetChannelsWithElevatedPrivileges() {
        AuthedAccount account = new AuthedAccount("john", "abc", false);

        assertTrue(account.getChannelsWithElevatedPrivileges().size() == 0);

        account.setChannelPriviledge("#lounge", ChannelPrivilege.OP);
        assertTrue(account.getChannelsWithElevatedPrivileges().size() == 1);
        assertTrue(account.getChannelsWithElevatedPrivileges().contains("#lounge"));

        account.setChannelPriviledge("#vip", ChannelPrivilege.OP);
        assertTrue(account.getChannelsWithElevatedPrivileges().size() == 2);
        assertTrue(account.getChannelsWithElevatedPrivileges().contains("#lounge"));
        assertTrue(account.getChannelsWithElevatedPrivileges().contains("#vip"));

        account.setChannelPriviledge("#prog", ChannelPrivilege.NONE);
        assertTrue(account.getChannelsWithElevatedPrivileges().size() == 2);
        assertTrue(account.getChannelsWithElevatedPrivileges().contains("#lounge"));
        assertTrue(account.getChannelsWithElevatedPrivileges().contains("#vip"));
    }
}
