package net.meltingwax.anonircd.tests;

import net.meltingwax.anonircd.*;
import net.meltingwax.anonircd.excepts.*;
import net.meltingwax.anonircd.i.*;
import static org.easymock.EasyMock.*;
import junit.framework.TestCase;


public class ChannelTest extends TestCase {

    public void testConstructor() {
        try {
            new Channel(null, null);
            fail();
        } catch (NullPointerException e) {
            // this is what we want
        } catch (IllegalChannelNameException e) {
            fail();
        }

        try {
            new Channel(null, "#abc");
        } catch (NullPointerException e) {
            // this is what we want
        } catch (IllegalChannelNameException e) {
            fail();
        }

        try {
            new Channel(createMock(ConfigProvider.class), null);
            fail();
        } catch (NullPointerException e) {
            // this is what we want
        } catch (IllegalChannelNameException e) {
            fail();
        }

        try {
            new Channel(createMock(ConfigProvider.class), "#lounge");
        } catch (IllegalChannelNameException e) {
            fail();
        }

        try {
            new Channel(createMock(ConfigProvider.class), "#lounge xyz");
            fail();
        } catch (IllegalChannelNameException e) {
            // this is what we want
        }
    }

    public void testIsValidChannelName() {
        try {
            Channel.isValidChannelName(null);
            fail();
        } catch (NullPointerException e) {
            // this is what we want
        }

        assertTrue(Channel.isValidChannelName("#"));
        assertTrue(Channel.isValidChannelName("#abc"));
        assertTrue(Channel.isValidChannelName("##math"));
        assertTrue(Channel.isValidChannelName("#math-lounge"));
        assertTrue(Channel.isValidChannelName("#math_lounge"));

        assertFalse(Channel.isValidChannelName(""));
        assertFalse(Channel.isValidChannelName(" #"));
        assertFalse(Channel.isValidChannelName("# "));
        assertFalse(Channel.isValidChannelName("#ab c"));
        assertFalse(Channel.isValidChannelName("#a bc"));
        assertFalse(Channel.isValidChannelName(" #abc"));
    }

    public void testAddClient_checksForIPBanPositive() {
        ConfigProvider configProvider = createMock(ConfigProvider.class);
        ChannelParticipant participant = createMock(ChannelParticipant.class);

        expect(participant.getIPAddress()).andReturn("1.2.3.4");
        expect(configProvider.checkForChannelBan("#lounge", "1.2.3.4")).andReturn(true);
        expect(configProvider.getChannelTopic("#lounge")).andReturn(null);
        
        replay(configProvider);
        replay(participant);

        Channel channel = null;
        try {
            channel = new Channel(configProvider, "#lounge");
        } catch (IllegalChannelNameException e) {
            fail();
        }

        try {
            channel.addClient(participant);
            fail();
        } catch (ClientIsBannedException e) {
            // this is what we want
        }

        verify(configProvider);
        verify(participant);
    }

    public void testAddClient_checksForIPBanNegative() {
        ConfigProvider configProvider = createMock(ConfigProvider.class);
        ChannelParticipant participant = createMock(ChannelParticipant.class);

        expect(participant.getIPAddress()).andReturn("1.2.3.4").atLeastOnce();
        expect(configProvider.checkForChannelBan("#lounge", "1.2.3.4")).andReturn(false);
        expect(configProvider.getChannelTopic("#lounge")).andReturn(null);

        replay(configProvider);
        replay(participant);

        Channel channel = null;
        try {
            channel = new Channel(configProvider, "#lounge");
        } catch (IllegalChannelNameException e) {
            fail();
        }

        try {
            channel.addClient(participant);
        } catch (ClientIsBannedException e) {
            fail();
        }

        verify(configProvider);
        verify(participant);
    }

    public void testAddClient_callsParticipantCallback() {
        Channel channel = null;
        ConfigProvider configProvider = createMock(ConfigProvider.class);
        ChannelParticipant participant1 = createMock(ChannelParticipant.class);
        ChannelParticipant participant2 = createMock(ChannelParticipant.class);

        expect(configProvider.checkForChannelBan("#lounge", "1.0.0.1")).andReturn(false).atLeastOnce();
        expect(configProvider.checkForChannelBan("#lounge", "2.0.0.2")).andReturn(false).atLeastOnce();
        expect(configProvider.getChannelTopic("#lounge")).andReturn(null).atLeastOnce();
        replay(configProvider);

        try {
            channel = new Channel(configProvider, "#lounge");
        } catch (IllegalChannelNameException e) {
            fail();
        }

        participant1.onJoinedChannelNewChannelParticipant(channel);
        assertEquals(0, channel.getNumParticipants());

        // ---------------
        expect(participant1.getIPAddress()).andReturn("1.0.0.1").atLeastOnce();
        expect(participant1.isParticipantAlive()).andReturn(true).atLeastOnce();
        expect(participant2.getIPAddress()).andReturn("2.0.0.2").atLeastOnce();
        expect(participant2.isParticipantAlive()).andReturn(true).atLeastOnce();
        replay(participant1);
        replay(participant2);
        
        try {
            channel.addClient(participant1);
        } catch (ClientIsBannedException e) {
            fail();
        }
        
        assertEquals(1, channel.getNumParticipants());

        try {
            channel.addClient(participant2);
        } catch (ClientIsBannedException e) {
            fail();
        }

        assertEquals(2, channel.getNumParticipants());

        verify(configProvider);
        verify(participant1);
        verify(participant2);
    }

    public void testRemoveClient_callParticipantCallback() {
        Channel channel = null;
        ConfigProvider configProvider = createMock(ConfigProvider.class);
        ChannelParticipant participant1 = createMock(ChannelParticipant.class);
        ChannelParticipant participant2 = createMock(ChannelParticipant.class);

        expect(configProvider.checkForChannelBan("#lounge", "1.0.0.1")).andReturn(false).atLeastOnce();
        expect(configProvider.checkForChannelBan("#lounge", "2.0.0.2")).andReturn(false).atLeastOnce();
        expect(configProvider.getChannelTopic("#lounge")).andReturn(null).atLeastOnce();
        expect(participant1.getIPAddress()).andReturn("1.0.0.1").atLeastOnce();
        expect(participant2.getIPAddress()).andReturn("2.0.0.2").atLeastOnce();
        expect(participant1.isParticipantAlive()).andReturn(true).atLeastOnce();
        expect(participant2.isParticipantAlive()).andReturn(true).atLeastOnce();
        replay(configProvider);
        
        try {
            channel = new Channel(configProvider, "#lounge");
        } catch (IllegalChannelNameException e) {
            fail();
        }

        participant1.onJoinedChannelNewChannelParticipant(channel);
        participant2.onJoinedChannelParticipantPart(channel);
        assertEquals(0, channel.getNumParticipants());

        replay(participant1);
        replay(participant2);
        

        try {
            channel.addClient(participant1);
        } catch (ClientIsBannedException e) {
            fail();
        }

        assertEquals(1, channel.getNumParticipants());

        try {
            channel.addClient(participant2);
        } catch (ClientIsBannedException e) {
            fail();
        }

        assertEquals(2, channel.getNumParticipants());
        channel.removeClient(participant1);
        assertEquals(1, channel.getNumParticipants());
        channel.removeClient(participant2);
        assertEquals(0, channel.getNumParticipants());

        verify(configProvider);
        verify(participant1);
        verify(participant2);
    }

    public void testRemoveClient_notInChannel() {
        Channel channel = null;
        ConfigProvider configProvider = createMock(ConfigProvider.class);
        ChannelParticipant participant1 = createMock(ChannelParticipant.class);
        ChannelParticipant participant2 = createMock(ChannelParticipant.class);

    
        expect(participant1.getIPAddress()).andReturn("1.0.0.1").atLeastOnce();
        expect(configProvider.checkForChannelBan("#lounge", "1.0.0.1")).andReturn(false);
        expect(configProvider.getChannelTopic("#lounge")).andReturn("");
        replay(configProvider);
        replay(participant1);
        replay(participant2);
        
        try {
            channel = new Channel(configProvider, "#lounge");
        } catch (IllegalChannelNameException e) {
            fail();
        }

        try {
            channel.addClient(participant1);
        } catch (ClientIsBannedException e) {
            fail();
        }

        try {
            channel.removeClient(participant2);
            fail();
        } catch (IllegalArgumentException e) {
            // this is what we want
        }

        channel.removeClient(participant1);

        verify(configProvider);
        verify(participant1);
        verify(participant2);
    }
}
